/*
 * Copyright (c) 2008 - 2012, Andy Bierman, All Rights Reserved.
 * Copyright (c) 2012 - 2020, YumaWorks, Inc., All Rights Reserved.
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 *

*** Generated by yangdump-sdk

    Combined SIL module
    module session-hook
    revision 2020-07-09
    namespace http://netconfcentral.org/ns/ses-hook
    Created: 2020-07-09T18:17:17Z
    CLI parameters:
        format c
        indent 4
        log-level debug3
        module session-hook
        output session-hook.c
        sil-edit2
        sil-get2
        unified true

 */

#include <xmlstring.h>

#include "procdefs.h"
#include "agt.h"
#include "agt_action.h"
#include "agt_cb.h"
#include "agt_cfg.h"
#include "agt_rpc.h"
#include "agt_sil_lib.h"
#include "agt_timer.h"
#include "agt_util.h"
#include "agt_val.h"
#include "cfg.h"
#include "dlq.h"
#include "getcb.h"
#include "ncx.h"
#include "ncx_feature.h"
#include "ncxmod.h"
#include "ncxtypes.h"
#include "obj.h"
#include "rpc.h"
#include "status.h"
#include "val.h"
#include "val_child.h"
#include "val_util.h"
#include "xml_val.h"
#include "session-hook.h"

/* module static variables */
static ncx_module_t *session_hook_mod;
static obj_template_t *interface_cont_obj;

/* put your static variables here */

/********************************************************************
* FUNCTION y_session_hook_init_static_vars
*
* initialize module static variables
*
********************************************************************/
static void y_session_hook_init_static_vars (void)
{
    session_hook_mod = NULL;
    interface_cont_obj = NULL;

    /* init your static variables here */

} /* y_session_hook_init_static_vars */


/********************************************************************
* FUNCTION session_hook_interface_cont_edit
*
* Edit database object callback
* Path: container /interface-cont
* Add object instrumentation in COMMIT phase.
*
* INPUTS:
*     see agt/agt_cb.h for details
*
* RETURNS:
*     error status
********************************************************************/
static status_t
    session_hook_interface_cont_edit (ses_cb_t *scb,
                                      rpc_msg_t *msg,
                                      agt_cbtyp_t cbtyp,
                                      op_editop_t editop,
                                      val_value_t *newval,
                                      val_value_t *curval)
{
    status_t res = NO_ERR;
    val_value_t *errorval = (curval) ? curval : newval;
    uint32 cur_errs = agt_get_error_count(msg);

    if (LOGDEBUG) {
        log_debug("\nEnter session_hook_interface_cont_edit callback for %s phase",
            agt_cbtype_name(cbtyp));
    }

    if (editop == OP_EDITOP_MERGE) {
        /* the edit is not really on this node; need to get
         * each child_undo record to get the real edited nodes */
        agt_cfg_transaction_t *txcb = RPC_MSG_TXCB(msg);
        agt_cfg_undo_rec_t *child_edit =
            agt_cfg_first_child_edit(txcb, newval, curval);

        while (child_edit) {
            op_editop_t child_editop = OP_EDITOP_NONE;
            val_value_t *child_newval = NULL;
            val_value_t *child_curval = NULL;

            agt_cfg_child_edit_fields(child_edit,
                &child_editop,
                &child_newval,
                &child_curval);

            if (LOGDEBUG2) {
                log_debug2("\nchild edit %s", op_editop_name(child_editop));
                if (child_newval) {
                    log_debug2_append("\nnew");
                    val_dump_value(child_newval, 0, DBG2);
                }
                if (child_curval) {
                    log_debug2_append("\ncur");
                    val_dump_value(child_curval, 0, DBG2);
                }
            }

            obj_template_t *chobj = (child_newval) ?
                VAL_OBJ(child_newval) : VAL_OBJ(child_curval);
            const xmlChar *name = obj_get_name(chobj);

            /**** process child edits here ****/

            if (!xml_strcmp(name, y_session_hook_N_admin_status_dyn)) {
                /* leaf admin-status-dyn (enumeration) */

            }

            child_edit = agt_cfg_next_child_edit(child_edit);
        }
    }

    switch (cbtyp) {
    case AGT_CB_VALIDATE:
        /* description-stmt validation here */
        break;
    case AGT_CB_APPLY:
        /* database manipulation done here */
        break;
    case AGT_CB_COMMIT:
        /* device instrumentation done here */
        switch (editop) {
        case OP_EDITOP_LOAD:
            break;
        case OP_EDITOP_MERGE:
            break;
        case OP_EDITOP_REPLACE:
            break;
        case OP_EDITOP_CREATE:
            break;
        case OP_EDITOP_DELETE:
            break;
        default:
            res = SET_ERROR(ERR_INTERNAL_VAL);
        }
        break;
    case AGT_CB_ROLLBACK:
        /* undo device instrumentation here */
        break;
    default:
        res = SET_ERROR(ERR_INTERNAL_VAL);
    }

    if ((res != NO_ERR) && (cur_errs == agt_get_error_count(msg))) {
        agt_record_error(scb,
                         &msg->mhdr,
                         NCX_LAYER_CONTENT,
                         res,
                         NULL,
                         (errorval) ? NCX_NT_VAL : NCX_NT_NONE,
                         errorval,
                         (errorval) ? NCX_NT_VAL : NCX_NT_NONE,
                         errorval);
    }
    return res;

} /* session_hook_interface_cont_edit */


/********************************************************************
* FUNCTION session_hook_callback
*
* The Session Hook callback is invoked when a session starts
* and ends.
*
* INPUTS:
*    ses_event == session event type
*    scb == session control block for the session event
*
* RETURNS:
*   status
********************************************************************/
static void
    session_hook_callback (ncx_ses_event_t ses_event,
                           const ses_cb_t *scb)
{
    if (LOGDEBUG) {
        log_debug("\n\nEnter Session Hook callback for %s@%s",
                 SES_MY_USERNAME(scb) ? SES_MY_USERNAME(scb) : NCX_EL_NOBODY,
                 SES_PEERADDR(scb) ? SES_PEERADDR(scb) : NCX_EL_LOCALHOST);
    }

    /* Perform specific tasks when the session is ended or started
     * and only if the current session protocol is RESTCONF
     * and the session has an input that is encoded in XML
     */
    ncx_display_mode_t input_mode = SES_IN_ENCODING(scb);
    if (input_mode == NCX_DISPLAY_MODE_XML) {

        if (SES_PROTOCOL(scb) == NCX_PROTO_RESTCONF) {
            if (ses_event == NCX_SES_EVENT_START) {

                if (LOGDEBUG) {
                    log_debug("\n ++++ RESTCONF Session Start ++++");
                }
            } else if (ses_event == NCX_SES_EVENT_END) {

                if (LOGDEBUG) {
                    log_debug("\n ---- RESTCONF Session END ----");
                }
            }
        }
    }

    return;

}  /* session_hook_callback */


/********************************************************************
* FUNCTION y_session_hook_init
*
* initialize the session-hook server instrumentation library
*
* INPUTS:
*    modname == requested module name
*    revision == requested version (NULL for any)
*
* RETURNS:
*     error status
********************************************************************/
status_t
    y_session_hook_init (const xmlChar *modname,
                         const xmlChar *revision)
{
    status_t res = NO_ERR;

    y_session_hook_init_static_vars();

    /* change if custom handling done */
    if (xml_strcmp(modname, y_session_hook_M_session_hook)) {
        return ERR_NCX_UNKNOWN_MODULE;
    }

    if (revision && xml_strcmp(revision, y_session_hook_R_session_hook)) {
        return ERR_NCX_WRONG_VERSION;
    }

    res =
        ncxmod_load_module(y_session_hook_M_session_hook,
                           y_session_hook_R_session_hook,
                           agt_get_savedevQ(),
                           &session_hook_mod);
    if (res != NO_ERR) {
        return res;
    }

    interface_cont_obj =
        ncx_find_object(session_hook_mod,
                        y_session_hook_N_interface_cont);
    if (interface_cont_obj == NULL) {
        return ERR_NCX_DEF_NOT_FOUND;
    }

    res =
        agt_cb_register_edit2_callback(y_session_hook_M_session_hook,
                                       (const xmlChar *)"/seshk:interface-cont",
                                       y_session_hook_R_session_hook,
                                       session_hook_interface_cont_edit);
    if (res != NO_ERR) {
        return res;
    }

    /* put your module initialization code here */

    /* Register a Session Hook callback */
    res =
        agt_cb_session_hook_register(session_hook_callback);
    if (res != NO_ERR) {
        return res;
    }

    return res;

} /* y_session_hook_init */

/********************************************************************
* FUNCTION y_session_hook_init2
*
* SIL init phase 2: non-config data structures
* Called after running config is loaded
*
* RETURNS:
*     error status
********************************************************************/
status_t y_session_hook_init2 (void)
{
    status_t res = NO_ERR;

    /* put your init2 code here */

    return res;

} /* y_session_hook_init2 */

/********************************************************************
* FUNCTION y_session_hook_cleanup
*    cleanup the server instrumentation library
*
********************************************************************/
void y_session_hook_cleanup (void)
{

    agt_cb_unregister_callbacks(y_session_hook_M_session_hook,
                                (const xmlChar *)"/seshk:interface-cont");

    /* put your cleanup code here */

    /* Unregister a Session Hook callback. */
    agt_cb_session_hook_unregister(session_hook_callback);

} /* y_session_hook_cleanup */

/* END SIL session_hook.c */
